"""
Class: SettingsBase
Description: Base settings for a vectorization profile
"""
from pydantic import BaseModel

class SettingsBase(BaseModel):
    """
    Base settings for a vectorization profile.
    """
    pass
